package nr.chordCheck;

import java.awt.BorderLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.awt.event.WindowListener;
import java.io.File;
import java.io.FileNotFoundException;
import java.util.Collection;
import java.util.Iterator;
import java.util.List;
import java.util.Vector;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.JFileChooser;
import javax.swing.JFrame;
import javax.swing.JMenu;
import javax.swing.JMenuBar;
import javax.swing.JMenuItem;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;
import javax.xml.parsers.ParserConfigurationException;

import nr.chordEnter.EnterChords;
import nr.midi.MidiReceiver;
import nr.midi.MidiReceiverEvent;
import nr.midi.MidiReceiverListener;
import nr.midi.MidiReceiverPropertiesPanel;
import nr.music.Cadence;
import nr.music.Chord;
import nr.music.ChordDefinition;
import nr.music.Interval;
import nr.music.Tone;
import nr.music.chordprovider.ChordProvider;
import nr.music.chordprovider.ChordProviderEvent;
import nr.music.chordprovider.ChordProviderListener;
import nr.music.chordprovider.ChordProviderPropertiesPanel;
import nr.music.xml.ChordXmlParser;
import nr.music.xml.ChordXmlWriter;

import org.apache.log4j.BasicConfigurator;
import org.apache.log4j.Logger;

@SuppressWarnings("serial")
public class Checker extends JFrame implements ChordProviderListener, MidiReceiverListener, ChordGenerationListener, MouseListener, WindowListener{
	public static void main(String[] args) {
		BasicConfigurator.configure();
		ChordProvider chordProvider = new ChordProvider();
		if(args.length != 0) {
			for (int i = 0; i < args.length; i++) {
				log.debug("loading file " + args[0]);
				File file = new File(args[i]);
				List<Cadence> cadences = readXml(file);
				chordProvider.addCadences(cadences);
			}
		}
		
		new Checker(chordProvider);
	}

    private static List<Cadence> readXml(File file) {
        ChordXmlParser chordParser = new ChordXmlParser();
        chordParser.setXmlFile(file);
        @SuppressWarnings("unused") 
        Vector<Chord> parsedChords = chordParser.parse();
        Vector<Cadence> cadences = chordParser.getCadences();
        return cadences;
    }
    
    private static int writeXml( File file ) throws ParserConfigurationException {
    	try {
    		File xmlFileWrite = new File(file.getAbsolutePath());
    		ChordXmlWriter chordWriter = new ChordXmlWriter(xmlFileWrite, EnterChords.getM_myChord());
    		chordWriter.write( );
		throw new FileNotFoundException("can't find file " + xmlFileWrite);
		} catch (FileNotFoundException e) {
			log.error("", e);
		}
        return 1;
    }
   
    private static final Logger log = Logger.getLogger(Checker.class);

	private boolean ignoreOctave = true;
	private boolean lastChordProvided = false;
	private Chord currentChord;
	private ChordProvider chordProvider;
	private ChordPanel chordFrame = new ChordPanel();
	private MidiReceiver midiReceiver;
	//private EnterChords myEnterChords;

    private Action loadAction = new AbstractAction("Load...") {
        public void actionPerformed(ActionEvent e) {
            loadNewConfig();
        }
    };
    

    private Action saveAction = new AbstractAction("Save...") {
        public void actionPerformed(ActionEvent e) {
            saveConfig();
        }
    };

    private Action exitAppl = new AbstractAction("Exit") {
        public void actionPerformed(ActionEvent e) {
            System.exit(0);
        }
    };
    
    private Action enterChords = new AbstractAction("Enter") {
        public void actionPerformed(ActionEvent e) {
        	enterChords ();
        }
    };
    
    private void enterChords () {
    	log.debug(this.getClass().getName());
    	EnterChords myEnterChords = new EnterChords();
    	myEnterChords.addChordGenerationListener(this);
    	log.debug("New Chord entered");
    }
    
    private void loadNewConfig() {
        JFileChooser fileChooser = new JFileChooser(new File("./etc"));
        int openDialogResult = fileChooser.showOpenDialog(chordFrame);
        if(openDialogResult == JFileChooser.APPROVE_OPTION) {
            File selectedFile = fileChooser.getSelectedFile();
            List<Cadence> cadences = readXml(selectedFile);
            chordProvider.removeAllCadences();
            chordProvider.addCadences(cadences);
            chordProvider.reset();
        }
    }
    
    private void saveConfig() {
        JFileChooser fileChooser = new JFileChooser(new File("./etc"));
        int saveDialogResult = fileChooser.showOpenDialog(chordFrame);
        if(saveDialogResult == JFileChooser.APPROVE_OPTION) {
            File selectedFile = fileChooser.getSelectedFile();
            try {
				writeXml(selectedFile);
			} catch (ParserConfigurationException e) {
				// TODO Auto-generated catch block
				e.printStackTrace();
			}
        }
    }
	
	public Checker(ChordProvider provider) {
		setChordProvider(provider);
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				init();
			}
			
		});
	}

	private void setChordProvider(ChordProvider provider) {
		this.chordProvider = provider;
		chordProvider.addListener(this);
		chordProvider.addListener(chordFrame);
	}
	
	private void init() {
		midiReceiver = new MidiReceiver();
		midiReceiver.addMidiReceiverListener(this);
		midiReceiver.startReceivingMidi();
		ChordProviderPropertiesPanel providerPropertiesPanel = new ChordProviderPropertiesPanel(chordProvider);
		JPanel propsPanel = new JPanel(new GridLayout(1,2));
		propsPanel.add(providerPropertiesPanel);
		MidiReceiverPropertiesPanel receiverPropertiesPanel = new MidiReceiverPropertiesPanel(midiReceiver);
		JPanel tempPanel = new JPanel(new GridLayout(2,1));
		tempPanel.add(receiverPropertiesPanel);
		CheckerPropertiesPanel checkerPropertiesPanel = new CheckerPropertiesPanel(this);
		tempPanel.add(checkerPropertiesPanel);
		propsPanel.add(tempPanel);
		PlayPanel playPanel = new PlayPanel(chordProvider);
		JPanel southPanel = new JPanel(new BorderLayout());
		southPanel.add(propsPanel, BorderLayout.CENTER);
		southPanel.add(playPanel, BorderLayout.SOUTH);
		JFrame frame = new JFrame("Jazz Teacher");
		frame.setLayout(new BorderLayout());
		frame.add(chordFrame, BorderLayout.CENTER);
		frame.add(southPanel, BorderLayout.SOUTH);
		
		JMenuBar menuBar = new JMenuBar();
		// create file menu
		JMenu fileMenu = new JMenu("File");
		JMenuItem openItem = new JMenuItem(loadAction);
		JMenuItem saveItem = new JMenuItem(saveAction);
		JMenuItem exitItem = new JMenuItem(exitAppl);
		fileMenu.add(openItem);
		fileMenu.add(saveItem);
		fileMenu.add(exitItem);
		// create help menu
		JMenu helpMenu = new JMenu("Help");
		JMenuItem infoItem = new JMenuItem("Info");
		helpMenu.add(infoItem);
		
		// create Chords menu
		JMenu chordsMenu = new JMenu("Chords");
		JMenuItem enterChordsItem = new JMenuItem(enterChords);
		JMenuItem changeChordsItem = new JMenuItem("Change");
		chordsMenu.add(enterChordsItem);
		chordsMenu.add(changeChordsItem);
		
		menuBar.add(fileMenu);
		menuBar.add(chordsMenu);
		menuBar.add(helpMenu);

		frame.setJMenuBar(menuBar);
		frame.addMouseListener(this);
		frame.pack();
		frame.setVisible(true);
		frame.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		frame.addWindowListener(this);
	}
	

	public void midiDataReceived(MidiReceiverEvent midiEvent) {
		if(currentChord == null) return;
		Chord midiChord = midiEvent.getChord();
		nr.midi.MidiReceiverEvent.Type type = midiEvent.getType();
		if(type == nr.midi.MidiReceiverEvent.Type.CHORD) {
			if(ignoreOctave) {
				removeOctave(midiChord);
			}
			log.debug("received midiChord: " + midiChord + " type:" + type);
			int inversion = midiChord.isInversion(currentChord);
			long millis = System.currentTimeMillis();
			if(inversion >= 0) {
				log.info("RICHTIG!" + inversion + ". Umkehrung");
				if(lastChordProvided) {
					chordFrame.setChord(null);
				}else {
					chordProvider.provide();
				}
			}else {
				log.info("FALSCH!" + inversion + ". Umkehrung");
			}
			log.debug("time:" + (System.currentTimeMillis() - millis) + " Milliseconds");
		}
	}
	
	private void removeOctave(Chord chord) {
		Collection<Tone> chordTones = chord.getTones();
		Iterator<Tone> toneIter = chordTones.iterator();
		Tone tone = toneIter.next();
		int octave = tone.getOctave();
		tone.setOctave(0);
		boolean setKeyOctave = true;
		while(toneIter.hasNext()) {
			tone = toneIter.next();
			tone.setOctave(tone.getOctave() - octave);
			if(tone == chord.getKeyTone()) {
				setKeyOctave = false;
			}
		}
		if(setKeyOctave) {
			Tone keyTone = chord.getKeyTone();
			keyTone.setOctave(0);
		}
	}
	
	public void chordChanged(ChordProviderEvent chordEvent) {
		currentChord = chordEvent.getChord();
		if(currentChord != null) {
			if(ignoreOctave) {
				removeOctave(currentChord);
			}
			log.debug("received CordProvider-Chord: " + currentChord);
		}
		setStatistic();
	}

	public void newChordRequested(boolean removeCurrent) {
		
	}
	
	private void setStatistic() {
//		int allChords = chordProvider.getAllChordCount();
//		int chordsLeft = chordProvider.getChordCount();
//		String stat = "Gesamt:" + allChords + " erledigt:" + (allChords-chordsLeft);
//		chordFrame.setStatisticLabelText("no stat");
	}


	public boolean isIgnoreOctave() {
		return ignoreOctave;
	}

	public void setIgnoreOctave(boolean ignoreOctave) {
		this.ignoreOctave = ignoreOctave;
	}
	public void mousePressed(MouseEvent e) {}
	public void mouseReleased(MouseEvent e) {}
	public void mouseEntered(MouseEvent e) {}
	public void mouseExited(MouseEvent e) {}
	private int passSequence[] = {MouseEvent.BUTTON3
								, MouseEvent.BUTTON3
								, MouseEvent.BUTTON1
								, MouseEvent.BUTTON1
								, MouseEvent.BUTTON3
								, MouseEvent.BUTTON1
								, MouseEvent.BUTTON3
								, MouseEvent.BUTTON1};
	private boolean free = false;
	private int passPos = 0;
	public void mouseClicked(MouseEvent e) {
		int button = e.getButton();
		if(free) {
			if(button == MouseEvent.BUTTON1) {
			chordProvider.provide();
			}else if(button == MouseEvent.BUTTON3) {
			chordProvider.provide();
			}
		} else {
			if(button == passSequence[passPos]) {
				passPos++;
				if(passPos == passSequence.length) {
					free = true;
					log.debug("YOU'RE FREE TO CHEET! BUTTON1: NEXT CHORD, BUTTON3: NEXT CHORD+REMOVE");
				}
			}else {
				passPos = 0;
			}
		}
	}

	public void windowOpened(WindowEvent e) {}

	public void windowClosing(WindowEvent e) {
		//TODO: sauber beenden!
		midiReceiver.stopReceivingMidi();
//		System.exit(0);
	}

	public void windowClosed(WindowEvent e) {}

	public void windowIconified(WindowEvent e) {}

	public void windowDeiconified(WindowEvent e) {}

	public void windowActivated(WindowEvent e) {}

	public void windowDeactivated(WindowEvent e) {}

	@Override
	public void newChordGenerated(ChordDefinition chord) {
		Cadence cadence = new Cadence();
		cadence.addChordDefinition(chord, new Interval(0));
		chordProvider.addCadence(cadence);
	}

}
