package nr.chordEnter;

import java.awt.Color;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.awt.event.WindowEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.JButton;
import javax.swing.JFrame;

import nr.chordCheck.Checker;
import nr.chordCheck.ChordGenerationListener;
import nr.music.ChordDefinition;
import nr.music.ChordSymbol;
import nr.music.ChordSymbol.Line;
import nr.music.Interval;
import nr.music.Tone;

import org.apache.log4j.Logger;

public class EnterChords extends JFrame implements MouseListener
{
	private static final long serialVersionUID = 1L;
	
    private static final Logger log = Logger.getLogger(Checker.class);

	private static int			MAX_NR_OF_NOTES		= 15;
	
	public final boolean		KEY_DOWN			= true;
	public final boolean		KEY_UP				= false;
	protected String[]			m_sArrID			= null;
	protected int				m_iOctave			= 0;
	private static boolean 		m_setBaseNoteFlag   = false;
	public int 					m_baseNoteIndex      = 0;

	private int					m_iKeyWidth			= 20;
	private int					m_iKeyLength		= 55;
	
	private Color				m_default_color		= Color.WHITE;
	private Color				m_current_color		= m_default_color;	
	private CleffPanel          m_chordPanel        = null;	
	private ConfPanel 			m_confPanel			= null;
	
	public static GenericNote[]	m_NoteQueue			= null;
	private int					m_iNoteCounter		= 0;
	private int					m_iLastNote			= 0;
	public  int					m_iTmpIterator		= 0;
	public static String	 	m_globalState		= "";
	public static ChordDefinition 	m_myChord		= null;
	public static ChordSymbol 		m_mySymbol		= null;
	
	public EnterChords(ChordDefinition chordDefinition, ChordSymbol chordSymbol ) {
		m_myChord = chordDefinition;
		m_mySymbol = chordSymbol;
		this.InitEnterChords();
	}
	
	public EnterChords() {
		m_mySymbol = new ChordSymbol("EnteredNewSymbol");
		m_myChord = new ChordDefinition("EnteredNewChord", m_mySymbol);
		this.InitEnterChords();
	}
		
	private void InitEnterChords() {
	
		m_NoteQueue = new GenericNote[MAX_NR_OF_NOTES];
	
		this.addMouseListener(this);	
		this.setUndecorated(false);
		this.setResizable(false);
		this.setDefaultCloseOperation(JFrame.DISPOSE_ON_CLOSE);
		this.setModalExclusionType(null);
		
		m_confPanel = new ConfPanel(this);	
		m_confPanel.setLocation(600,0);
		m_chordPanel = new CleffPanel(this);
		m_chordPanel.addMouseListener(null);
//		m_chordPanel.setLocation(300, 300);
		m_chordPanel.setVisible(true);
		
		this.setLayout(new GridBagLayout());
		GridBagConstraints c = new GridBagConstraints();
		
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0; 
		c.gridy = 0;
		c.gridwidth = 2;
		this.add(m_chordPanel, c); 
		
		JButton jOk = new JButton("Close");
		JButton jSave = new JButton("Save");
		JButton jRevert = new JButton("Revert");
		
		jOk.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0)
			{
				EnterChords.this.setVisible(false);				
				EnterChords.this.dispatchEvent(new WindowEvent(EnterChords.this, WindowEvent.WINDOW_CLOSED));
			}			
		});
	
		jSave.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0) {
				convertNotesToChord();
				notifyListener(m_myChord);
			}			
		});
		
		jRevert.addActionListener(new ActionListener(){
			public void actionPerformed(ActionEvent arg0)
			{
				//jRevert functionality
				
			}			
		});
						
		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 0; 
		c.gridy = 1;	
		c.gridwidth = 1;
		this.add(jOk, c);

		c.fill = GridBagConstraints.HORIZONTAL;
		c.gridx = 1; 
		c.gridy = 1;
		c.gridwidth = 2;
		this.add(jSave, c);
		this.pack();
		this.setVisible(true);				
	}
	
	public ChordDefinition convertNotesToChord() {	
		
		// conversion function to fill chord structure correctly
		int iInterval;
		
		log.debug("m_iNoteCounter: " + m_iNoteCounter);
		
		// only debug output for now-> all notes from cleff panel
		for (int i=0; i<m_iNoteCounter; i++) {
			log.debug("Note index: " + i);
			log.debug("m_NoteQueue[i].m_noteName: " + m_NoteQueue[i].m_noteName+m_NoteQueue[i].m_sAccidental[0]);
			log.debug("m_NoteQueue[i].m_iOctave: " + m_NoteQueue[i].m_iOctave);
			log.debug("m_NoteQueue[i].m_noteName: " + m_NoteQueue[i].m_noteName);		
		}

		for (int i=0; i<m_iNoteCounter; i++) {
			//if base note found
			if (i==m_baseNoteIndex) {
//				Interval interval = new Interval(0);
//				m_myChord.addInterval(interval);
				continue;
			}
				
			iInterval = lookIntervalFromBaseNote(m_NoteQueue[m_baseNoteIndex], m_NoteQueue[i] );
			
			if (iInterval != -1 ) {
				Interval interval = new Interval(iInterval);
				m_myChord.addInterval(interval);
			}			
		}		
		
		m_mySymbol.addText(Line.BASE, m_confPanel.getM_textFieldBaseLine().getText(), true);			
		
		m_mySymbol.addRef(Line.BASE, "signlesskey", false);
		m_mySymbol.addRef(Line.UPPER, "sign", true);	

		m_mySymbol.addText(Line.UPPER, m_confPanel.getM_textFieldUpperLine().getText(), true);
		m_mySymbol.setName(m_confPanel.getM_textFieldChordName().getText());
		m_myChord.setName( m_confPanel.getM_textFieldChordLabel() );
		
		return m_myChord;
	}
	
	public int getLowestNoteIndex( ) {
		int lowestIndex=0;	
		for (int i=1; i< m_iNoteCounter; i++) {
			if (m_NoteQueue[i].m_iStepsFromC < m_NoteQueue[lowestIndex].m_iStepsFromC) {
				lowestIndex=i;
			}		
		}
		
		log.debug("lowestIndex of note:" + lowestIndex);
		return lowestIndex;		
	}
	
	public static int getNoteIndex( GenericNote note ) {
		String sConvName = note.m_noteName + note.m_sAccidental[0] ;		
		
		if ( note.m_sAccidental[0] == "b" || note.m_sAccidental[0]=="" ) {
			for( int i=0; i< Tone.keyNamesFlat.length; i++) {
				if (sConvName.equals(Tone.keyNamesFlat[i])) {
					return i;
				}
			}
		}	
		
		if ( note.m_sAccidental[0] == "#" ) {
			for( int i=0; i< Tone.keyNamesSharp.length; i++) {
				if (sConvName.equals(Tone.keyNamesSharp[i])) {
					return i;
				}
			}
		}

		if (sConvName.equals("Fb"))  {
			log.debug("Fb handled");
			return 4;	
		}
		
		if (sConvName.equals("Cb")) {
			log.debug("Cb handled");
			return -1;
		}	
		
		if (sConvName.equals("E#")) {
			log.debug("E# handled!");
			return 5;
		}
		
		if (sConvName.equals("B#")) {
			log.debug("B# handled!");
			return 12;
		}	
		
		return 0; 
	}

	private int lookIntervalFromBaseNote( GenericNote baseNote, GenericNote note ) {			
		log.debug("iInterval = " + (note.m_iStepsFromC - baseNote.m_iStepsFromC));	
		if ((note.m_iStepsFromC - baseNote.m_iStepsFromC)<0) {
			log.debug("ERROR: Interval lower than zero!!!");
			return 0;
		}
		return (note.m_iStepsFromC - baseNote.m_iStepsFromC);
	}	
	
	public void addNote(GenericNote newNote)
	{
		/* test if note already is existing -> yes, then remove */
		for (int i=0; i<m_iNoteCounter; i++)
		{
			if ( (newNote.m_iOctave == m_NoteQueue[i].m_iOctave) &&  
					(newNote.m_noteName.equals(m_NoteQueue[i].m_noteName) ) && 
						(newNote.m_bassClef == m_NoteQueue[i].m_bassClef )) {
				log.debug("Note already available, not added!");
				log.debug("Remove note!");
				removeNote(newNote);
				//set lowest note as base note automatically
				setBaseNoteIndex(getLowestNoteIndex());
				return;	
			}
		}
		
		//add note to queue
		newNote.setNoteIndex(m_iNoteCounter);
		m_NoteQueue[m_iNoteCounter] = newNote;
		m_iLastNote=m_iNoteCounter;
		m_NoteQueue[m_iNoteCounter].m_sAccidental[0]=m_globalState;
//		int currentIndex=m_iNoteCounter;
		m_iNoteCounter++;
				
		//treat the base note
//		if ( isM_setBaseNoteFlag() == true) {		
			//first test, if note is the lowest note of all;
			//if yes, set new base note, if not, just add as normal note
//			if (getLowestNoteIndex() == currentIndex ) {
//				setBaseNoteIndex (currentIndex);
//			}			
//		} else {
			//set lowest note as base note automatically
			setBaseNoteIndex(getLowestNoteIndex());
//		}		

		log.debug("set m_baseNoteIndex: " + m_baseNoteIndex);		
		
		/*  Increase array if some people add too many notes */
		if (m_iNoteCounter >= m_NoteQueue.length)
		{		
			GenericNote[] iTmpArray = new GenericNote[m_NoteQueue.length + MAX_NR_OF_NOTES ];

			for (int i = 0; i < m_NoteQueue.length; i++)
				iTmpArray[i] = m_NoteQueue[i];
			m_NoteQueue = iTmpArray;
			
			log.debug("Array increased");		
		}
	}

	public int getBaseNoteIndex() {
		return m_baseNoteIndex;
	}
	
	public void setBaseNoteIndex( int index ){
		m_baseNoteIndex = index;
	}

	/*  remove the note out of m_noteQueue if already existing in array  */
	public void removeNote(GenericNote targetNote)
	{
		int indexToRemove=-1;
		
		for (int i = 0; i < m_iNoteCounter; i++) {		
			if ( (targetNote.m_iOctave == m_NoteQueue[i].m_iOctave) &&  
					(targetNote.m_noteName == m_NoteQueue[i].m_noteName ) && 
						(targetNote.m_bassClef == m_NoteQueue[i].m_bassClef )) {
				log.debug("Note to remove has been found!");
				indexToRemove=i;
				break;
			}
		}
		log.debug("Index to remove: " + indexToRemove);
		/* finally remove */
		if (indexToRemove != -1 ) {
			for (int i = indexToRemove; i < m_iNoteCounter; i++) {
				m_NoteQueue[i]=m_NoteQueue[i+1];			
			}
			m_NoteQueue[m_iNoteCounter]=null;
			m_iNoteCounter--;
		}
	}
	
	/*  Retrieve next non-played note starting from the "top note" marker  */
	public GenericNote nextNote()
	{
		for (int i = m_iTmpIterator; i < m_iNoteCounter; i++)
			if (m_NoteQueue[i] != null )
			{
				m_iTmpIterator = i + 1;
				return m_NoteQueue[i];
			}

		return null;
	}
	
	public int getNotesCounter()
	{
		return m_iNoteCounter;
	}
	
	public int getLastNoteIndex()
	{
		return m_iLastNote;
	}
	
	public void draw(Graphics2D g) {
		log.debug("Draw 2D");
	}
	public void press_key()
	{
		color_key(KEY_DOWN);				
	}

	public void release_key()
	{
		color_key(KEY_UP);
	}

	public int getKeyWidth()
	{
		return m_iKeyWidth;
	}

	public int getKeyHeight()
	{
		return m_iKeyLength;
	}

	protected void setKeyWidth(int iWidth)
	{
		m_iKeyWidth = iWidth;
	}

	protected void setKeyHeight(int iHeight)
	{
		m_iKeyLength = iHeight;
	}

	protected void setCurrentColor(Color newColor)
	{
		m_current_color = newColor;
	}

	protected void setDefaultColor(Color newColor)
	{
		m_default_color = newColor;
	}

	protected Color getCurrentColor()
	{
		return m_current_color;
	}

	protected Color getDefaultColor()
	{
		return m_default_color;
	}

	private void color_key(boolean bKeyState)
	{
		if (bKeyState == KEY_DOWN)
		{
			m_current_color = Color.cyan;
		}
		else
		{
			m_current_color = m_default_color;
		}

		this.repaint();
	}

	public void paint(Graphics g)
	{
		super.paint(g);

		drawKey(g);
	}

	public void drawKey(Graphics g)
	{
	}
	
	public void setNoneSymbol() {
		if (m_NoteQueue[m_iLastNote] != null ) {
			m_NoteQueue[m_iLastNote].m_sAccidental[0]="";
		}
	}
	
	public void setFlatSymbol() {
		if (m_NoteQueue[m_iLastNote] != null ) {
			m_NoteQueue[m_iLastNote].m_sAccidental[0]="b";
		}
	}
	
	public void setSharpSymbol() {
		if (m_NoteQueue[m_iLastNote] != null ) {		
			m_NoteQueue[m_iLastNote].m_sAccidental[0]="#";
		}
	}
	
	public void mouseClicked(MouseEvent arg0)
	{
		//KeyBoard kb = (KeyBoard)this.getParent();
		//kb.PlayNote(new GenericNote(this.m_sArrID, this.m_iOctave));	
		log.debug("MouseClicked");
		log.debug("X: " + arg0.getX());
		log.debug("Y: " + arg0.getY());
		
	    //m_note = new GenericNote();
		GenericNote newNote = new GenericNote();	    
		m_chordPanel.calcNoteFromPosition(newNote, arg0.getX(), arg0.getY());
		
		addNote( newNote );
			    
	}

	public void mouseEntered(MouseEvent arg0)
	{		
		//log.debug("MouseEntered");
		//if ( (arg0.getModifiers() & MouseEvent.BUTTON1_MASK) == MouseEvent.BUTTON1_MASK)
	}

	public void mouseExited(MouseEvent arg0)
	{
		release_key();
	}

	public void mousePressed(MouseEvent arg0)
	{
		press_key();
		log.debug("MousePressed");
	}

	public void mouseReleased(MouseEvent arg0)
	{
		release_key();
	}
	List<ChordGenerationListener> listeners = new ArrayList<ChordGenerationListener>();
	public void addChordGenerationListener(ChordGenerationListener listener) {
		listeners.add(listener);
	}
	
	public void removeChordGenerationListener(ChordGenerationListener listener) {
		listeners.remove(listener);
	}
	
	private void notifyListener(ChordDefinition chordDefinition) {
		for (ChordGenerationListener listener : listeners) {
			listener.newChordGenerated(chordDefinition);
		}
	}

	public static void setM_setBaseNoteFlag(boolean m_setBaseNoteFlag) {
		EnterChords.m_setBaseNoteFlag = m_setBaseNoteFlag;
	}

	public static boolean isM_setBaseNoteFlag() {
		return m_setBaseNoteFlag;
	}

	public static ChordSymbol getM_mySymbol() {
		return m_mySymbol;
	}

	public void setM_mySymbol(ChordSymbol symbol) {
		m_mySymbol = symbol;
	}

	public static ChordDefinition getM_myChord() {
		return m_myChord;
	}

	public void setM_myChord(ChordDefinition chord) {
		m_myChord = chord;
	}
}
