package nr.chordEnter;

import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.image.BufferedImage;
import java.io.File;
import java.io.IOException;
import java.util.Comparator;

import javax.imageio.ImageIO;

import nr.chordCheck.Checker;

import org.apache.log4j.Logger;

public class GenericNote implements Comparator<GenericNote> {
	private static final Logger log = Logger.getLogger(Checker.class);

	public static final int NORMAL_NOTE = 0;
	public static final int BASE_NOTE = 1;

	public static final int m_baseNoteIndex = 0;

	/* Pixel positions of current note on window */
	public int m_iXPos;
	private int m_iYPos;

	/* all the information of the current note */
	public String m_noteName; /* name of note */
	public String[] m_sNoteID = null; /*
									 * note value itself as String
									 * representation
									 */
	public int m_iOctave;
	public String[] m_sAccidental; /* empty, "b" or "#" right now */
	public int m_bassClef = 0; /* 1 equals bass cleff */
	public int m_iType; /* NORMAL_NOTE=0, BASE_NOTE=1; */

	/* steps from middle C: positive or negative :range -128 < x < 128 */
	public int m_iStepsFromC = 0;

	/* Horizontal index position of current note: TODO! */
	private int m_iIndex = 0;

	/* Contains note image */
	private BufferedImage m_note_img = null;

	public String[] m_keyArray = { "C", "D", "E", "F", "G", "A", "B", "C" };

	/**
	 * All default notes are born as middle C
	 */
	public GenericNote() {
		init(0, new String[] { "C" }, 1, new String[] { "" });
	}

	public GenericNote(String[] sNoteID, int iOctave) {
		init(0, sNoteID, iOctave, new String[] { "" });
	}

	public GenericNote(int iType, String[] sNoteID, int iOctave) {
		init(iType, sNoteID, iOctave, new String[] { "" });
	}

	public GenericNote(int iType, String[] sNoteID, int iOctave,
			String[] sAccidental) {
		init(iType, sNoteID, iOctave, sAccidental);
	}

	/* TODO: write more init functions for setting notes properly! */

	/**
	 * Initialize note variables
	 * 
	 * @param iType
	 * @param sID
	 * @param iOctave
	 */
	private void init(int iType, String[] sNoteID, int iOctave,
			String[] sAccidental) {
		this.m_iType = iType;
		this.m_sNoteID = sNoteID;
		this.m_iOctave = iOctave;
		this.m_sAccidental = sAccidental;
		this.m_iXPos = 0; /* TODO!! */

		try {
			if (iType == BASE_NOTE) {
				m_note_img = ImageIO.read(new File(
						"data/img/whole_note_base.gif"));
			} else { // NORMAL_NOTE
				m_note_img = ImageIO.read(new File("data/img/whole_note.gif"));
			}
		} catch (IOException e) {
			log.error("Error loading GIF image [whole_note.gif]", e);
		}
	}

	/**
	 * Get type of note: BASE_NOTE or NORMAL_NOTE
	 * 
	 * @return
	 */
	public int getNoteType() {
		int iType;
		if (EnterChords.isM_setBaseNoteFlag() == true) {
			iType = BASE_NOTE;
		} else {
			iType = NORMAL_NOTE;
		}
		return iType;
	}

	/**
	 * Set order that note appears visibly
	 * 
	 * @param iIndex
	 */
	public void setNoteIndex(int iIndex) {
		this.m_iIndex = iIndex;
	}

	/**
	 * Get order that note appears visibly
	 * 
	 * @return
	 */
	public int getNoteIndex() {
		return m_iIndex;
	}

	/**
	 * Set how far up or down this note should be from middle C
	 * 
	 * @param iStepsFromC
	 *            Can be positive or negative
	 */
	public void setStepsFromC(int iStepsFromC) {
		m_iStepsFromC = iStepsFromC;
	}

	/**
	 * How far up or down are we from middle C?
	 * 
	 * @return Can be positive or negative
	 */
	public int getStepsFromC() {
		return m_iStepsFromC;
	}

	/**
	 * Get Y pixel coordinates of upper left corner of note
	 * 
	 * @return
	 */
	public int getYPos() {
		return m_iYPos;
	}

	/**
	 * Get X pixel coordinates of upper left corner of note
	 * 
	 * @return
	 */
	public int getXPos() {
		return m_iXPos;
	}

	/**
	 * Set X pixel coordinates of upper left corner of note
	 * 
	 * @param iXPos
	 */
	public void setXPos(int iXPos) {
		m_iXPos = iXPos;
	}

	/**
	 * Set Y pixel coordinates of upper left corner of note
	 * 
	 * @param iYPos
	 */
	public void setYPos(int iYPos) {
		m_iYPos = iYPos;
	}

	/**
	 * Draw note image according to positional parameters and GameVars class
	 * properties.
	 * 
	 * @param g
	 */
	public void draw(Graphics2D g) {
		int iUpperLimit = 96; // (208 - 7*16); //??? TODO: relative calculations
		int iLowerLimit = 280;

		// log.debug("m_iYPos: " + m_iYPos);

		int iDiff_up = iUpperLimit - m_iYPos;
		// int iDiff_down = m_iYPos - iLowerLimit;

		// log.debug("iDiff_up: " + iDiff_up);
		// log.debug("iUpperLimit: " + iUpperLimit);
		// log.debug("iDiff_down: " + iDiff_down);
		// log.debug("iLowerLimit: " + iLowerLimit);

		/* Draw extra staff lines if note is beyond standard lines */
		if (iDiff_up > 0) {
			int i = iUpperLimit;
			while (i >= m_iYPos + 8) {
				g.drawLine(getXPos() - 15, i, getXPos() + 40, i);
				i -= 16;
			}
		}

		if (m_iYPos >= iLowerLimit) {
			int i = iLowerLimit;
			while (i <= m_iYPos) {
				g.drawLine(getXPos() - 15, i + 8, getXPos() + 40, i + 8);
				i += 16;
			}
		}

		/* Draw extra staff line if note is middle C */
		if ((m_iOctave == 0) && (m_noteName == "C") && (m_bassClef == 0))
			g.drawLine(getXPos() - 15, m_iYPos + 8, getXPos() + 40, m_iYPos + 8);

		/* Draw note */
		g.drawImage(m_note_img, getXPos(), m_iYPos, null);

		/* Draw sharp and flat labels */
		g.setFont(new Font("Serif", Font.BOLD | Font.ITALIC, 32));
		g.drawString(m_sAccidental[0], getXPos() - 20, getYPos() + 16);

	}

	public BufferedImage getImg() {
		return m_note_img;
	}

	public String[] getAccidental() {
		return this.m_sAccidental;
	}

	public int getOctave() {
		return m_iOctave;
	}

	public int compare(GenericNote noteA, GenericNote noteB) {
		/* TODO */
		return 0;
	}

	@Override
	/**
	 * Not all notes are created equal!
	 */
	public boolean equals(Object objNote) {
		GenericNote note = (GenericNote) objNote;

		/*
		 * Loop to explore String array of foreign note for sharp and flat
		 * symbols (#/b)
		 */
		for (int i = 0; i < note.m_sAccidental.length; i++) {
			for (int j = 0; j < this.m_sAccidental.length; j++) {
				if (note.m_sAccidental[i]
						.equalsIgnoreCase(this.m_sAccidental[j])
						&& (note.m_iOctave == this.m_iOctave || note.m_iOctave == -1)) {
					return true;
				}
			}
		}

		return false;
	}

}
