package nr.chordEnter;

import java.awt.Canvas;
import java.awt.Color;
import java.awt.Component;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.GraphicsEnvironment;
import java.awt.Image;
import java.awt.Toolkit;
import java.awt.Transparency;
import java.awt.color.ColorSpace;
import java.awt.image.BufferedImage;
import java.awt.image.BufferedImageOp;
import java.awt.image.ColorConvertOp;
import java.lang.reflect.Method;

/**
 * General utilities
 * 
 * @author Timbo
 *
 */
public class Utils
{	
//	public static byte NOTE_FLAT  = 2;
//	public static byte NOTE_SHARP = 4;	
	
	/**
	 * Color supplied buffered image 'newColor'.  
	 * @param bi Image to use as source
	 * @param newColor Color to transform image into
	 * @return Colored image
	 */
	public static BufferedImage toColor(BufferedImage bi, Color newColor) 
	{ 			 
		 int w = bi.getWidth();
		 int h = bi.getHeight();
		 int pixel;
		 int newPixel = newColor.getRGB();
		 
		 BufferedImage biOut = GraphicsEnvironment.getLocalGraphicsEnvironment().getDefaultScreenDevice().getDefaultConfiguration().createCompatibleImage(w, h, Transparency.BITMASK);
		 
		 for (int x=0; x < w; x++)
		 {
			 for( int y = 0; y < h; y ++)
			 {
				 pixel = bi.getRGB(x,y);
				 
				 if ((pixel & 0xff000000) != 0x00)
				 {
					 biOut.setRGB(x,y,newPixel);
				 }
			 }
		 }
	     	     
	     return biOut;
	}
	
	/**
	 * Convert supplied buffered image to its black and white version
	 * @param source
	 * @return Black and white representation
	 */
	public static BufferedImage toGrayscale(BufferedImage source) 
	{ 		
	     BufferedImageOp op = new ColorConvertOp(ColorSpace.getInstance(ColorSpace.CS_GRAY), null);
	     	     
	     return op.filter(source, null);
	}
	
	/**
	 * Utility for drawing images onto Canvas objects 
	 * 
	 * This is used to draw images onto JDialog displays
	 * 
	 * @param img
	 * @return
	 */
	public static Canvas drawImg(Image img)
	{
		class myCanvas extends Canvas
		{
			private static final long	serialVersionUID	= 1L;
			
			Image img = null;
			public myCanvas(Image img)
			{
				super();
				this.img = img;
				this.setBounds(0, 0, img.getWidth(null), img.getHeight(null));
			}
			
			@Override
			public void paint(Graphics g)
			{
				super.paint(g);
				
				g.drawImage(this.img, 0, 0, null);
			}
		}
		
		myCanvas canvas = new myCanvas(img);
		canvas.setBounds(0, 0, img.getWidth(null),	img.getHeight(null));
		return (Canvas)canvas;
	}
	
	public static void reCenter(Component child, Component parent) 
	{
		Dimension parentDim = (parent == null ? Toolkit.getDefaultToolkit().getScreenSize() : parent.getSize());
				
		int x = (int) (parentDim.getWidth() - child.getWidth()) / 2;
		int y = (int) (parentDim.getHeight() - child.getHeight()) / 2;
				
		child.setLocation(x, y);
	}
	
	/**
	 * Open HTML page with web browsers: Firefox, Opera, Konquereor, Epiphany, Mozilla and Netscape
	 * 
	 * Hopefully, the Linux user uses one of the hardcoded options listed below!
	 * 
	 * @param sURL
	 */
	public static void openURL(String sURL)
	{
		String osName =  System.getProperty("os.name");
		try
		{
			if ( osName.startsWith("Mac OS") )
			{
				Class <?> fileMgr = Class.forName("com.apple.eio.FileManager");
				
				Method openURL = fileMgr.getDeclaredMethod("openURL", new Class[] {String.class});
				
				openURL.invoke(null, new Object[] {sURL});
			}
			else if ( osName.startsWith("Windows") )
			{
				Runtime.getRuntime().exec("rundll32 url.dll, FileProtocolHandler " + sURL);
			}
			else
			{
				/*  Unix or Linux  */
				String [] allBrowsers = {"firefox", "opera", "konqueror", "epiphany", "mozilla", "netscape"};
				
				String sBrowser = null;
				
				for (int iCounter = 0; iCounter < allBrowsers.length && sBrowser == null; iCounter++)
				{
					if (Runtime.getRuntime().exec(new String[] {"which", allBrowsers[iCounter]}).waitFor() == 0)
							sBrowser = allBrowsers[iCounter];
					
					if ( sBrowser == null )
						throw new Exception("Could not find web browser");
					else
						Runtime.getRuntime().exec(new String[] {sBrowser, sURL});
				}
			}
		}
		catch (Exception e)
		{
			System.out.println("Exception encountered attempting to open your default web browser!" + System.getProperty("line.separator") + e.getMessage());
		}				
	}
}