package nr.midi;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.WindowAdapter;
import java.awt.event.WindowEvent;

import javax.sound.midi.MidiDevice;
import javax.sound.midi.MidiDevice.Info;
import javax.sound.midi.MidiSystem;
import javax.sound.midi.MidiUnavailableException;
import javax.sound.midi.Receiver;
import javax.sound.midi.Transmitter;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JPanel;
import javax.swing.SwingUtilities;

import org.apache.log4j.Logger;

public class MidiMain implements ItemListener {
	static private Logger log = Logger.getLogger(MidiMain.class);
	private JFrame frame;
	private JComboBox<Info> deviceCombo;
	private MidiDevice device;

	public MidiMain() {
		start();
	}

	public static void main(String[] args) {
		SwingUtilities.invokeLater(new Runnable() {
			public void run() {
				new MidiMain();
			}
		});
	}

	private void start() {
		frame = new JFrame();
		deviceCombo = new JComboBox<Info>();
		Info[] info = MidiSystem.getMidiDeviceInfo();
		log.debug("info.length:" + info.length);
		for (int i = 0; i < info.length; i++) {
			Info inf = info[i];
			log.debug("inf.getName():" + inf.getName() + "inf.getDescription()"
					+ inf.getDescription() + "###" + inf);
			deviceCombo.addItem(inf);
		}
		deviceCombo.addItemListener(this);
		frame.setLayout(new BorderLayout());
		JPanel panel = new JPanel();
		panel.setLayout(new FlowLayout());
		panel.add(deviceCombo);
		frame.add(panel);
		frame.setLocationRelativeTo(null);
		frame.pack();
		frame.addWindowListener(new LocalWindowListener());
		frame.setVisible(true);
	}

	public void itemStateChanged(ItemEvent e) {
		if (e.getSource() == deviceCombo) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				Info info = (Info) deviceCombo.getSelectedItem();
				hereIsYourDeviceInfoDoWhat(info);
			}
		}
	}

	private void hereIsYourDeviceInfoDoWhat(Info info) {
		log.debug("try to monitor mididevice:" + info);
		try {
			device = MidiSystem.getMidiDevice(info);
			device.open();
			log.debug("device-Class:" + device.getClass().getName());
			Transmitter transmitter = device.getTransmitter();
			Receiver receiver = new MidiReceiver();
			transmitter.setReceiver(receiver);
		} catch (MidiUnavailableException e) {
			log.info("can't read from that device");
			device.close();
		}
		log.debug("hereIsYourDeviceInfoDoWhat finished");

	}

	private void endApplication() {
		frame.setVisible(false);
		synchronized (this) {
			notifyAll();
		}
		System.exit(0);
	}

	class LocalWindowListener extends WindowAdapter {
		@Override
		public void windowClosing(WindowEvent e) {
			endApplication();
		}
	}

}
