package nr.midi;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.event.FocusEvent;
import java.awt.event.FocusListener;
import java.awt.event.ItemEvent;
import java.awt.event.ItemListener;
import java.awt.event.KeyAdapter;
import java.awt.event.KeyEvent;
import java.util.Vector;

import javax.sound.midi.MidiDevice.Info;
import javax.swing.JComboBox;
import javax.swing.JFrame;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;
import javax.swing.JTextField;
import javax.swing.border.TitledBorder;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

@SuppressWarnings("serial")
public class MidiReceiverPropertiesPanel extends JPanel implements
		ChangeListener, ItemListener, FocusListener {

	public static void main(String[] args) {
		MidiReceiver m = new MidiReceiver();
		MidiReceiverPropertiesPanel propPanel = new MidiReceiverPropertiesPanel(
				m);
		JFrame frame = new JFrame();
		frame.add(propPanel);
		frame.pack();
		frame.setVisible(true);
	}

	private MidiReceiver receiver;
	private JSlider toleranceSlider;
	private JTextField toleranceText;
	private JComboBox<Info> deviceCombo;
	private int maxTolerance = 1000;
	private int minTolerance = 0;

	public MidiReceiverPropertiesPanel(MidiReceiver receiver) {
		this.receiver = receiver;
		init();
	}

	private void init() {
		setLayout(new GridBagLayout());
		setBorder(new TitledBorder("Midi"));
		long toleranceMillis = receiver.getToleranceMillis();
		int row = -1;

		Vector <Info> deviceList = receiver.getDeviceList();
		deviceCombo = new JComboBox<Info>(deviceList);
		deviceCombo.addItemListener(this);
		row++;
		add(new JLabel("Midi Device"), new GridBagConstraints(0, row, 1, 1, 0,
				0, GridBagConstraints.WEST, GridBagConstraints.NONE,
				new Insets(0, 0, 0, 0), 10, 0));
		add(deviceCombo, new GridBagConstraints(1, row, 1, 1, 0, 0,
				GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(0,
						0, 0, 0), 10, 0));

		toleranceSlider = new JSlider(0, 1000, (int) (toleranceMillis));
		toleranceSlider.addChangeListener(this);
		toleranceSlider.setMajorTickSpacing(maxTolerance / 10);
		toleranceSlider.setMinorTickSpacing(maxTolerance / 100);
		toleranceSlider.setPaintTicks(true);
		// TODO: seems not to work???
		// toleranceSlider.setPaintLabels(true);
		// toleranceSlider.createStandardLabels(200);
		toleranceSlider.setPreferredSize(new Dimension(maxTolerance / 4, 30));
		JPanel sliderPan = new JPanel(new FlowLayout(FlowLayout.CENTER));
		sliderPan.add(new JLabel("0 ms"));
		sliderPan.add(toleranceSlider);
		sliderPan.add(new JLabel("1 s"));
		toleranceText = new JTextField("" + toleranceMillis);
		toleranceText.addFocusListener(this);
		toleranceText.setColumns(("" + maxTolerance).length());
		toleranceText.addKeyListener(new KeyAdapter() {
			public void keyTyped(KeyEvent e) {
				if (e.getKeyChar() == KeyEvent.VK_ENTER) {
					setToleranceTextToSlider();
				}
			}
		});
		sliderPan.add(toleranceText);
		sliderPan.add(new JLabel("ms"));
		row++;
		add(new JLabel("Tolerance"), new GridBagConstraints(0, row, 1, 1, 0, 0,
				GridBagConstraints.WEST, GridBagConstraints.NONE, new Insets(0,
						0, 0, 0), 10, 0));
		add(sliderPan, new GridBagConstraints(1, row, 1, 1, 0, 0,
				GridBagConstraints.WEST, GridBagConstraints.BOTH, new Insets(0,
						0, 0, 0), 10, 0));
	}

	public void stateChanged(ChangeEvent e) {
		if (e.getSource() == toleranceSlider) {
			long value = (long) toleranceSlider.getValue();
			receiver.setToleranceMillis(value);
			toleranceText.setText("" + value);
		}
	}

	public void itemStateChanged(ItemEvent e) {
		if (e.getSource() == deviceCombo) {
			if (e.getStateChange() == ItemEvent.SELECTED) {
				receiver.setDevice((Info) e.getItem());
			}
		}
	}

	public void focusGained(FocusEvent e) {
	}

	public void focusLost(FocusEvent e) {
		if (e.getSource() == toleranceText) {
			setToleranceTextToSlider();
		}
	}

	private void setToleranceTextToSlider() {
		try {
			int newVal = new Integer(toleranceText.getText()).intValue();
			if (newVal > maxTolerance) {
				newVal = maxTolerance;
			} else if (newVal < minTolerance) {
				newVal = minTolerance;
			}
			toleranceText.setBackground(getBackground());
			toleranceSlider.setValue(newVal);
		} catch (NumberFormatException ex) {
			toleranceText.setBackground(Color.RED);
		}
	}
}
