package nr.music;

import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;

public class Cadence {
	static private Logger log = Logger.getLogger(Cadence.class);
	Vector<StufenDefinition> stufen = new Vector<StufenDefinition>();
	int currentIdx = 0;
	
	public Cadence() {
	}
	
	public Vector<Chord> getConcretChords(Tone key) {
		Vector<Chord> chords = new Vector<Chord>();
		Iterator<StufenDefinition> iter = stufen.iterator();
		while (iter.hasNext()) {
			StufenDefinition stufenDef = iter.next();
			ChordDefinition definition = stufenDef.getDefinition();
			Interval interval = stufenDef.getInterval();
			log.debug("key:" + key + ", interval:" + interval);
			Tone nextKey = new Tone(key);
			nextKey.setAbsolutHeight(nextKey.getAbsolutHeight() + interval.getHalfTones());
			chords.add(definition.createChord(nextKey, key));
		}
		return chords;
	}
	
	/**
	 * @param definition - the ChordDefinition
	 * @param interval - the distance to the keyNote
	 */
	public void addChordDefinition(ChordDefinition definition, Interval interval) {
		stufen.add(new StufenDefinition(definition, interval));
	}
	
	/**
	 * @param definition - the ChordDefinition
	 * @param interval - distance to the keyNote
	 */
	public void removeChordDefinition(ChordDefinition definition, Interval interval) {
		stufen.remove(new StufenDefinition(definition, interval));
	}
	
	private class StufenDefinition{
		private ChordDefinition definition;
		private Interval interval;
		public StufenDefinition(ChordDefinition definition, Interval interval) {
			this.definition = definition;
			this.interval = interval;
		}
		public ChordDefinition getDefinition() {
			return definition;
		}
		public Interval getInterval() {
			return interval;
		}
		@Override
		public boolean equals(Object obj) {
			if(obj == null || !(obj instanceof StufenDefinition)) {
				return false;
			}
			StufenDefinition that = (StufenDefinition) obj;
			if(definition != null && definition.equals(that.definition)
					&& interval != null && interval.equals(that.interval)) {
				return true;
			}
			return false;
		}
	}
	
	@Override
	public boolean equals(Object obj) {
		if(obj == null || !(obj instanceof Cadence)) {
			return false;
		}
		Cadence that = (Cadence) obj;
		int thisSize = stufen.size();
		if(thisSize != that.stufen.size()) {
			return false;
		}
		for (int i = 0; i < thisSize; i++) {
			if(!stufen.get(i).equals(that.stufen.get(i))) {
				return false;
			}
		}
		return true;
	}
	
	@Override
	public String toString() {
		return "Cadence[ChordDefinitionCount:" + stufen.size() + "]";
	}
}
