package nr.music;

import java.awt.TextField;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;

public class ChordDefinition {
	static private Logger log = Logger.getLogger(ChordDefinition.class);
	
	private Vector<Interval> m_intervals = new Vector<Interval>();
	private Vector<Integer>  m_allowedInversions = new Vector<Integer>();
	
	ChordSymbol symbol;
	String name;
	
	public ChordDefinition(String name, ChordSymbol symbol) {
		this.name = name;
		this.symbol = symbol;
	}
	
	public void addInterval(Interval interval) {
		m_intervals.add(interval);
	}
	
	public Vector<Interval> getM_intervals() {
		return m_intervals;
	}

	public void setAllowedInversions(Vector<Integer> allowedInversions) {
		this.m_allowedInversions = allowedInversions;
	}
	
	public Vector<Integer> getM_allowedInversions() {
		return m_allowedInversions;
	}

	public Chord createChord(Tone keyTone) {
		return  createChord(keyTone, 0, null);
	}
	public Chord createChord(Tone keyTone, Tone cadenceKey) {
		return  createChord(keyTone, 0, cadenceKey);
	}
	public Chord createChord(Tone keyTone, int inversion) {
		return createChord(keyTone, inversion, null);
	}
	public Chord createChord(Tone keyTone, int inversion, Tone cadenceKey) {
		inversion = inversion % m_intervals.size();
		if(!m_allowedInversions.contains(new Integer(inversion))) {
			log.warn("inversion " + inversion + " not allowed");
		}
		Chord chord = new Chord();
		Iterator<Interval> iter = m_intervals.iterator();
		while (iter.hasNext()) {
			Interval interval = iter.next();
			int halfTones = interval.getHalfTones();
			Tone transposedTone = new Tone(keyTone.recalcAbsolutHeight() + halfTones);
			if(inversion > 0) {
				transposedTone.setOctave(transposedTone.getOctave() + 1);
				inversion--;
			}
			chord.addTone(transposedTone);
		}
		chord.setKeyTone(keyTone);
		chord.setCadenceKeyTone(cadenceKey);
		chord.setSymbol(new ChordSymbol(symbol));
		if(m_allowedInversions.size() > 0) {
			chord.setAllowedInversions(m_allowedInversions);
		}
		return chord;
	}
	
	public ChordSymbol getSymbol() {
		return symbol;
	}
	
	public String getName() {
		return name;
	}
	
	public void setName(TextField textField) {
		this.name=textField.getText();
	}
	
}
