package nr.music;

import java.awt.Color;
import java.awt.Font;
import java.awt.Graphics2D;
import java.awt.font.FontRenderContext;
import java.awt.font.TextLayout;
import java.awt.geom.Rectangle2D;
import java.awt.image.BufferedImage;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Vector;

import org.apache.log4j.Logger;

public class ChordSymbol {
	static private Logger log = Logger.getLogger(ChordSymbol.class);
	public enum Line {UPPER, BASE};
	private Vector<SymbolElement> m_symbolElements = new Vector<SymbolElement>();
	private String name;
	private Color foreGroundColor = Color.BLACK;
	private Color backGroundColor = Color.LIGHT_GRAY;


	public ChordSymbol(String name) {
		this.name = name;
	}
	
	public ChordSymbol(ChordSymbol symbol) {
		if (symbol== null) {
			log.error("Error: symbol name is a null pointer!");
			this.name = new String("EMTY");
		} else {
			this.name = symbol.name;
		}
		this.foreGroundColor = symbol.foreGroundColor;
		this.backGroundColor = symbol.backGroundColor;
		Iterator<SymbolElement> iter = symbol.m_symbolElements.iterator();
		while (iter.hasNext()) {
			SymbolElement element = iter.next();
			SymbolElement newElement = null; 
			if(element instanceof SymbolRefElement) {
				newElement = new SymbolRefElement(element.m_text, element.m_line, element.m_smallTextSize);
			}else {
				newElement = element;
			}
			m_symbolElements.add(newElement);
		}
	}

	public void addText(Line line, String text, boolean smallText) {
		log.debug("add Element to symbols");
		m_symbolElements.add(new SymbolElement(text, line, smallText));
	}
	public void addRef(Line line, String ref, boolean smallText) {
		log.debug("add Element to symbols");
		m_symbolElements.add(new SymbolRefElement(ref, line, smallText));
	}
	public void setRefValue(String refName, String value) {
		log.debug("add Element to symbols.  refName:" + refName + ", value:" + value);
		Iterator<SymbolElement> iter = m_symbolElements.iterator();
		while (iter.hasNext()) {
			SymbolElement element = iter.next();
			if(element instanceof SymbolRefElement) {
				SymbolRefElement refElement = (SymbolRefElement)element;
				refElement.setRefValue(refName, value);
			}
		}
	}

	private BufferedImage createImage() {
		log.debug("creating image...");
		BufferedImage tempImage = new BufferedImage(2800, 800, BufferedImage.TYPE_INT_RGB);
		Graphics2D g = tempImage.createGraphics();
		int height = tempImage.getHeight();
		int width = tempImage.getWidth();
		g.setColor(backGroundColor );
        g.fillRect(0,0,width,height);
		g.setColor(foreGroundColor );
		int mainFontSize = (int)(height);
		int smallFontSize = (int)(mainFontSize * 0.4);
		Font bigFont = new Font("Arial", Font.BOLD, mainFontSize);
		g.setFont(bigFont);
		Font smallFont = new Font("Arial", Font.BOLD, smallFontSize);
		g.setFont(smallFont);
		Iterator<SymbolElement> iter = m_symbolElements.iterator();
		log.debug("m_symbolElements.size():" + m_symbolElements.size());
		int x = 0;
		int third = height/3;
		int yBase = height - third + (height/10);
		int yUpper = third;
		while (iter.hasNext()) {
			SymbolElement symbolElement = iter.next();
			String symbolElementText = symbolElement.getText();
			if(symbolElementText != null && symbolElementText.length() > 0) {
				g.setFont(symbolElement.isSmallTextSize() ? smallFont:bigFont);			
				switch (symbolElement.getLine()) {
				case BASE:
					log.debug("drawString(), symbolText:" + symbolElementText + ", x:" + x + ", yBase:" + yBase);
					//x = getNextX(x, yBase);
					g.drawString(symbolElementText, x, yBase);
					break;
				case UPPER:
					log.debug("drawString(), symbolText:" + symbolElementText + ", x:" + x + ", yUpper:" + yUpper);
					//x = getNextX(x, yUpper);
					g.drawString(symbolElementText, x, yUpper);
					break;
				}
				FontRenderContext renderContext = g.getFontRenderContext();
				TextLayout tl = new TextLayout(symbolElementText, g.getFont(), renderContext);
				Rectangle2D rec = tl.getBounds();
				log.debug("symbolElementText:" + symbolElementText + ", textbreite:" + rec.getWidth());
				x = x + (int)(rec.getWidth() * 1.1);
			}
		}
		BufferedImage newImage = new BufferedImage(x, tempImage.getHeight(), BufferedImage.TYPE_INT_RGB);
		newImage.setData(tempImage.getData());
		log.debug("image created.");
		return newImage;
	}
	
//	private int getNextX(int startX, int maxY) {
//		int newX = startX;
//		int rgb = foreGroundColor.getRGB();
//		int red = (rgb >> 16) & 0xff;
//		int green = (rgb >> 8) & 0xff;
//		int blue = (rgb) & 0xff;
//		for (int iX = startX; iX > 0; iX--) {
//			boolean foundPixel = false;
//			for (int iY = 0; iY < maxY; iY++) {
//				float[] dA = new float[3];
////				dA = image.getData().getPixel(iX,iY,dA);
//				if(dA[0] == red && dA[1] == green && dA[2] == blue) {
//					foundPixel = true;
//					break;
//				}
//			}
//			if(foundPixel) {
//				newX = iX + 1;
//				break;
//			}
//		}
//		return newX;
//	}
	
	
	
	public BufferedImage getSymbolImage() {
		return createImage();
	}
	
	public void setBackGroundColor(Color backGroundColor) {
		if(this.backGroundColor.equals(backGroundColor)) {
			return;
		}
		this.backGroundColor = backGroundColor;
	}
	
	public void setForeGroundColor(Color foreGroundColor) {
		if(this.foreGroundColor.equals(foreGroundColor)) {
			return;
		}
		this.foreGroundColor = foreGroundColor;
	}

	public String getName() {
		return name;
	}
	
	public void setName( String chordName ) {
		this.name = chordName;
	}

	public class SymbolElement{
		
		Line m_line;
		boolean m_smallTextSize = false;
		//TODO: public needed for ChordXMLWriter, still to be changed
		public String m_text;
		
		public SymbolElement(String text, Line line, boolean smallText) {
			this.m_text = text;
			this.m_line = line;
			this.m_smallTextSize = smallText;
		}

		public Line getLine() {
			return m_line;
		}

		public boolean isSmallTextSize() {
			return m_smallTextSize;
		}

		public String getText() {
			return m_text;
		}
	}
	
	
	public class SymbolRefElement extends SymbolElement{
		private HashMap<String, String> m_referencesTexts = new HashMap<String, String>();

		public SymbolRefElement(String text, Line line, boolean smallText) {
			super(text, line, smallText);
		}
		public String getText() {
			 return m_referencesTexts.get(m_text);
		}
		
		public String getRefText() {
			 return this.getText();
		}
		
		public void setRefValue(String ref, String value){
			m_referencesTexts.put(ref, value);
		}
		public HashMap<String, String> getM_referencesTexts() {
			return m_referencesTexts;
		}
		
	}
	
	@Override
	public String toString() {
		return "ChordSymbol[name:" + name + "]";
	}

	public Vector<SymbolElement> getM_symbolElements() {
		return m_symbolElements;
	}

}
