package nr.music.chordprovider;

import java.util.ArrayList;

import nr.music.Cadence;

import org.apache.log4j.Logger;

public class CadenceProvider {
	private static final Logger log = Logger.getLogger(CadenceProvider.class);
	public enum Mode {SEQUENTIAL, RANDOM};
	private Mode mode = Mode.SEQUENTIAL;
	
	private ArrayList<Cadence> cadenceList = new ArrayList<Cadence>();
	private ArrayList<Cadence> allCadencesList = new ArrayList<Cadence>();
	
	public CadenceProvider() {
	}
	
	public void reset() {
		log.debug("reset...");
		cadenceList.clear();
		cadenceList.addAll(allCadencesList);
		resort();
		log.debug("...reset");
	}
	
	public void resort() {
		log.debug("resort... mode:" + mode);
		ArrayList<Cadence> tempList = new ArrayList<Cadence>(cadenceList);
		cadenceList.clear();
		if (mode == Mode.SEQUENTIAL) {
			for (Cadence cadence : allCadencesList) {
				if(tempList.contains(cadence)) {
					log.debug("adding cadence " + cadence);
					cadenceList.add(cadence);
				}
			}
		} else {
			int tempListSize;
			while ((tempListSize = tempList.size()) > 0) {
				int nextIdx = (int) (Math.random() * tempListSize);
				Cadence nextCadence = tempList.remove(nextIdx);
				log.debug("adding cadence to list:" + nextCadence);
				cadenceList.add(nextCadence);
			}
		}
		log.debug("...resort");
	}
	
	public Cadence next() {
		Cadence next = null;
		if (cadenceList.size() > 0) {
			next = cadenceList.remove(0);
		}
		log.debug("next:" + next);
		return next;
	}

	public Mode getMode() {
		return mode;
	}

	public void setMode(Mode mode) {
		if(this.mode != mode) {
			this.mode = mode;
			resort();
		}
	}
	
	public void addCadence(Cadence cadence) {
		allCadencesList.add(cadence);
	}
	public void removeCadence(Cadence cadence) {
		allCadencesList.remove(cadence);
	}
	public void removeAllCadences() {
		allCadencesList.clear();
	}

}
