package nr.music.chordprovider;

import java.util.ArrayList;
import java.util.Iterator;
import java.util.List;

import nr.music.Cadence;
import nr.music.Chord;
import nr.music.Tone;
import nr.music.chordprovider.KeyProvider.KeySequence;
import nr.music.chordprovider.KeyProvider.SignMode;

import org.apache.log4j.Logger;

public class ChordProvider {
	private static final Logger log = Logger.getLogger(ChordProvider.class);
	private ArrayList<ChordProviderListener> listeners = new ArrayList<ChordProviderListener>();
	private KeyProvider keyProvider = new KeyProvider();
	private CadenceProvider cadenceProvider = new CadenceProvider();
	private ArrayList<Chord> chordsToProvide = new ArrayList<Chord>();
	private ArrayList<Chord> allreadyProvided = new ArrayList<Chord>();
	private boolean autoStart;
	private boolean random;
	
	public ChordProvider() {
	}

	public void provide() {
		Chord nextChord = null;
		if(chordsToProvide.size() == 0 && autoStart) {
			reset();
		}
		if(chordsToProvide.size() != 0) {
			nextChord = chordsToProvide.remove(0);
			allreadyProvided.add(nextChord);
		}
		notifyChordChanged(nextChord, chordsToProvide.size() == 0);
	}
	
	private ArrayList<Chord> readAllChords() {
		ArrayList<Chord> allChords = new ArrayList<Chord>();
		keyProvider.reset();
		Tone key = null;
		while((key = keyProvider.next()) !=  null) {
			Cadence nextCadence = null;
			cadenceProvider.reset();
			while((nextCadence = cadenceProvider.next()) != null) {
				allChords.addAll(nextCadence.getConcretChords(key));
			} 
		}
		return allChords;
	}

	public void reset() {
		log.debug("reset...");
		allreadyProvided.clear();
		resort();
		notifyChordChanged(null, false);
	}
	
	private void resort() {
		log.debug("resort...");
		chordsToProvide.clear();
		ArrayList<Chord> allChords = readAllChords();
		allChords.removeAll(allreadyProvided);
		if(random) {
			int size;
			while((size = allChords.size()) > 0) {
				int rnd = (int)(Math.random() * size);
				chordsToProvide.add(allChords.remove(rnd));
			}
		} else {
			chordsToProvide = allChords;
		}
	}

	public void addListener(ChordProviderListener listener) {
		listeners.add(listener);
	}
	public void removeListener(ChordProviderListener listener) {
		listeners.remove(listener);
	}
	private void notifyChordChanged(Chord chord, boolean isLast) {
		ChordProviderEvent event = new ChordProviderEvent(chord, isLast);
		Iterator<ChordProviderListener> iter = listeners.iterator();
		log.debug("notifying listeners:" + event);
		while (iter.hasNext()) {
			ChordProviderListener listener = iter.next();
			listener.chordChanged(event);
		}
	}

	//Delegationmethods
	public KeySequence getKeySequence() {
		return keyProvider.getKeySequence();
	}
	public void setKeySequence(KeySequence keySequence) {
		keyProvider.setKeySequence(keySequence);
		resort();
	}
	public nr.music.chordprovider.CadenceProvider.Mode getCadenceProviderMode() {
		return cadenceProvider.getMode();
	}
	public void setCadenceProviderMode(nr.music.chordprovider.CadenceProvider.Mode mode) {
		cadenceProvider.setMode(mode);
		resort();
	}
	public SignMode getSignMode() {
		return keyProvider.getSignMode();
	}
	public void setSignMode(SignMode signMode) {
		keyProvider.setSignMode(signMode);
		for (Chord chord : chordsToProvide) {
			Tone keyTone = chord.getKeyTone();
			keyTone.setSign(signMode.getSign());
		}
	}
	public void addDisabledKey(Tone key) {
		keyProvider.addDisabledKey(key);
		resort();
	}
	public void removeDisabledKey(Tone key) {
		keyProvider.removeDisabledKey(key);
		resort();
	}
	public void addCadence(Cadence cadence) {
		cadenceProvider.addCadence(cadence);
		resort();
	}
	public void addCadences(List<Cadence> cadences) {
		for (Cadence cadence : cadences) {
			cadenceProvider.addCadence(cadence);
		}
		resort();
	}
	public void removeCadence(Cadence cadence) {
		cadenceProvider.removeCadence(cadence);
		resort();
	}
	public void removeAllCadences() {
		cadenceProvider.removeAllCadences();
		resort();
	}
	public void setAutoStart(boolean autoStart) {
		this.autoStart = autoStart;
	}
	public void setRandom(boolean random) {
		this.random = random;
	}
}
