package nr.music.chordprovider;

import java.awt.BorderLayout;
import java.awt.FlowLayout;
import java.awt.GridLayout;
import java.awt.event.ActionEvent;
import java.util.ArrayList;

import javax.swing.AbstractAction;
import javax.swing.Action;
import javax.swing.ButtonGroup;
import javax.swing.JCheckBox;
import javax.swing.JPanel;
import javax.swing.JRadioButton;
import javax.swing.SwingConstants;
import javax.swing.border.TitledBorder;

import nr.music.Tone;
import nr.music.chordprovider.CadenceProvider.Mode;
import nr.music.chordprovider.KeyProvider.KeySequence;
import nr.music.chordprovider.KeyProvider.SignMode;

import org.apache.log4j.Logger;

@SuppressWarnings("serial")
public class ChordProviderPropertiesPanel extends JPanel{
	static private Logger log = Logger.getLogger(ChordProviderPropertiesPanel.class);
	
	private ChordProvider chordProvider;

	private Action keyModeSequentialAction = new AbstractAction("Sequential") {
		public void actionPerformed(ActionEvent e) {
			if(keySequenceChromaticButton.isSelected()) {
				chordProvider.setKeySequence(KeySequence.CHROMATIC);
			} else if (keySequenceQuartcircleButton.isSelected()) {
				chordProvider.setKeySequence(KeySequence.QUARTCIRCLE);
			} else if (keySequenceQuintcircleButton.isSelected()) {
				chordProvider.setKeySequence(KeySequence.QUINTCIRCLE);
			}
			setKeySequenceButtonsEnabled();
		}
	};
	private Action allRandomAction = new AbstractAction("all Random") {
		public void actionPerformed(ActionEvent e) {
			JCheckBox box = (JCheckBox) e.getSource();
			chordProvider.setRandom(box.isSelected());
			if(box.isSelected()) {
				setKeyModeButtonsEnabled(false);
				setCadenceButtonsEnabled(false);
				setKeySequenceButtonsEnabled(false);
			} else {
				setKeyModeButtonsEnabled(true);
				setCadenceButtonsEnabled(true);
				setKeySequenceButtonsEnabled();
			}
		}
	};
	
	private Action keyModeRandomAction = new AbstractAction("Random") {
		public void actionPerformed(ActionEvent e) {
			chordProvider.setKeySequence(KeySequence.RANDOM);
			setKeySequenceButtonsEnabled();
		}
	};
	private Action keySequenceChromaticAction = new AbstractAction("Chromatic") {
		public void actionPerformed(ActionEvent e) {
			chordProvider.setKeySequence(KeySequence.CHROMATIC);
		}
	};
	private Action keySequenceQuartcircleAction = new AbstractAction("Quartcircle") {
		public void actionPerformed(ActionEvent e) {
			chordProvider.setKeySequence(KeySequence.QUARTCIRCLE);
		}
	};
	private Action keySequenceQuintcircleAction = new AbstractAction("Quintcircle") {
		public void actionPerformed(ActionEvent e) {
			chordProvider.setKeySequence(KeySequence.QUINTCIRCLE);
		}
	};
	private Action signFlatAction = new AbstractAction("b") {
		public void actionPerformed(ActionEvent e) {
			chordProvider.setSignMode(SignMode.SIGN_FLAT);
		}
	};
	private Action signSharpAction = new AbstractAction("#") {
		public void actionPerformed(ActionEvent e) {
			chordProvider.setSignMode(SignMode.SIGN_SHARP);
		}
	};
	
	private Action signRandomAction = new AbstractAction("Random") {
		public void actionPerformed(ActionEvent e) {
			chordProvider.setSignMode(SignMode.SIGN_RANDOM);
		}
	};
	
	private Action selectAllKeysAction = new AbstractAction("all") {
		public void actionPerformed(ActionEvent e) {
			Object source = e.getSource();
			if(source instanceof JCheckBox) {
				JCheckBox cb = (JCheckBox) source;
				boolean setSelected = cb.isSelected();
				for (KeyCheckBox keyCheckBox : keyCheckBoxes) {
					keyCheckBox.getCheckBox().setSelected(setSelected);
					Tone tone = keyCheckBox.getTone();
					setKeysToProvider(tone, setSelected);
				}
			}
		}
	};
	private Action cadenceSequentielAction = new AbstractAction("Sequential") {
		public void actionPerformed(ActionEvent e) {
			chordProvider.setCadenceProviderMode(Mode.SEQUENTIAL);
		}
	};
	private Action cadenceRandomAction = new AbstractAction("Random") {
		public void actionPerformed(ActionEvent e) {
			chordProvider.setCadenceProviderMode(Mode.RANDOM);
		}
	};
	
	private ArrayList<KeyCheckBox> keyCheckBoxes = new ArrayList<KeyCheckBox>();

	private JRadioButton keySequenceChromaticButton;
	private JRadioButton keySequenceQuartcircleButton;
	private JRadioButton keySequenceQuintcircleButton;
	private JRadioButton cadenceSequentielButton;
	private JRadioButton cadenceRandomButton;
	private JRadioButton keyModeSequentialButton;
	private JRadioButton keyModeRandomButton;
	
	public ChordProviderPropertiesPanel(ChordProvider chordProvider) {
		setBorder(new TitledBorder("Provider Configuration"));
		this.chordProvider = chordProvider;
		for(int i = 0; i < 12; i++) {
			keyCheckBoxes.add(new KeyCheckBox(new Tone(i)));
		}
		setLayout(new BorderLayout());
		JPanel keyProvidingPanel = createKeyProvidingPanel();
		JPanel cadencePanel = createCadenceProvidingPanel();
		
		JPanel providingPanel = new JPanel(new BorderLayout());
		providingPanel.setBorder(new TitledBorder(""));
		
		JCheckBox box = new JCheckBox(allRandomAction);
		providingPanel.add(box, BorderLayout.NORTH);
		providingPanel.add(keyProvidingPanel, BorderLayout.CENTER);
		providingPanel.add(cadencePanel, BorderLayout.SOUTH);
		
		add(providingPanel, BorderLayout.CENTER);
		JPanel signPanel = createSignPanel();
		JPanel keyPanel = createKeyPanel();
		JPanel southPanel = new JPanel(new GridLayout(2,1));
		southPanel.add(keyPanel);
		southPanel.add(signPanel);
		add(southPanel, BorderLayout.SOUTH);
	}

	private JPanel createSignPanel() {
		JPanel signPanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		signPanel.setBorder(new TitledBorder("Sign"));
		ButtonGroup signGroup = new ButtonGroup();		
		
		JRadioButton signFlatButton = new JRadioButton(signFlatAction);
		signFlatButton.setHorizontalTextPosition(SwingConstants.LEADING);
		signGroup.add(signFlatButton);
		signPanel.add(signFlatButton);

		JRadioButton signSharpButton = new JRadioButton(signSharpAction);
		signSharpButton.setHorizontalTextPosition(SwingConstants.LEADING);
		signGroup.add(signSharpButton);
		signPanel.add(signSharpButton);
		
		JRadioButton signRandomButton = new JRadioButton(signRandomAction);
		signRandomButton.setHorizontalTextPosition(SwingConstants.LEADING);
		signGroup.add(signRandomButton);
		signPanel.add(signRandomButton);
		
		switch (chordProvider.getSignMode()) {
		case SIGN_FLAT:
			signFlatButton.setSelected(true);
			break;
		case SIGN_SHARP:
			signSharpButton.setSelected(true);
			break;
		case SIGN_RANDOM:
			signRandomButton.setSelected(true);
			break;
		}
		return signPanel;
	}

	private JPanel createCadenceProvidingPanel() {
		JPanel cadencePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		cadencePanel.setBorder(new TitledBorder("Cadenceproviding"));
		ButtonGroup cadenceModeGroup = new ButtonGroup();
		
		cadenceSequentielButton = new JRadioButton(cadenceSequentielAction);
		cadenceSequentielButton.setHorizontalTextPosition(SwingConstants.LEADING);
		cadenceModeGroup.add(cadenceSequentielButton);
		cadencePanel.add(cadenceSequentielButton);

		cadenceRandomButton = new JRadioButton(cadenceRandomAction);
		cadenceRandomButton.setHorizontalTextPosition(SwingConstants.LEADING);
		cadenceModeGroup.add(cadenceRandomButton);
		cadencePanel.add(cadenceRandomButton);

		switch (chordProvider.getCadenceProviderMode()) {
		case SEQUENTIAL:
			cadenceSequentielButton.setSelected(true);
			break;
		case RANDOM:
			cadenceRandomButton.setSelected(true);
			break;
		}
		return cadencePanel;
	}

	private JPanel createKeyProvidingPanel() {
		JPanel keyProvidingPanel = new JPanel(new GridLayout(3,1));
		keyProvidingPanel.setBorder(new TitledBorder("Keyproviding"));
		
		JPanel modePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		keyProvidingPanel.add(modePanel);
		ButtonGroup modeGroup = new ButtonGroup();		
		
		keyModeSequentialButton = new JRadioButton(keyModeSequentialAction);
		keyModeSequentialButton.setHorizontalTextPosition(SwingConstants.LEADING);
		modeGroup.add(keyModeSequentialButton);
		modePanel.add(keyModeSequentialButton);

		keyModeRandomButton = new JRadioButton(keyModeRandomAction);
		keyModeRandomButton.setHorizontalTextPosition(SwingConstants.LEADING);
		modeGroup.add(keyModeRandomButton);
		modePanel.add(keyModeRandomButton);
		switch (chordProvider.getCadenceProviderMode()) {
		case SEQUENTIAL:
			keyModeSequentialButton.setSelected(true);
			break;
		case RANDOM:
			keyModeRandomButton.setSelected(true);
			break;
		}

		JPanel keySequencePanel = new JPanel(new FlowLayout(FlowLayout.LEFT));
		keyProvidingPanel.add(keySequencePanel);
		ButtonGroup keySequenceGroup = new ButtonGroup();		
		
		keySequenceChromaticButton = new JRadioButton(keySequenceChromaticAction);
		keySequenceChromaticButton.setHorizontalTextPosition(SwingConstants.LEADING);
		keySequenceGroup.add(keySequenceChromaticButton);
		keySequencePanel.add(keySequenceChromaticButton);

		keySequenceQuartcircleButton = new JRadioButton(keySequenceQuartcircleAction);
		keySequenceQuartcircleButton.setHorizontalTextPosition(SwingConstants.LEADING);
		keySequenceGroup.add(keySequenceQuartcircleButton);
		keySequencePanel.add(keySequenceQuartcircleButton);

		keySequenceQuintcircleButton = new JRadioButton(keySequenceQuintcircleAction);
		keySequenceQuintcircleButton.setHorizontalTextPosition(SwingConstants.LEADING);
		keySequenceGroup.add(keySequenceQuintcircleButton);
		keySequencePanel.add(keySequenceQuintcircleButton);
		switch (chordProvider.getKeySequence()) {
		case CHROMATIC:
			keySequenceChromaticButton.setSelected(true);
			break;
		case QUARTCIRCLE:
			keySequenceQuartcircleButton.setSelected(true);
			break;
		case QUINTCIRCLE:
			keySequenceQuintcircleButton.setSelected(true);
			break;
		case RANDOM:
			//ignore
			break;
		}
		
		return keyProvidingPanel;
	}

	private JPanel createKeyPanel() {
		JPanel keyPanel = new JPanel(new FlowLayout(FlowLayout.CENTER));
		keyPanel.setBorder(new TitledBorder("Keys"));
		for (KeyCheckBox keyCheckBox : keyCheckBoxes) {
			keyPanel.add(keyCheckBox.getCheckBox());
		}
		JCheckBox selectAllKeysButton = new JCheckBox(selectAllKeysAction);
		keyPanel.add(selectAllKeysButton);
		return keyPanel;
	}
	
	private void setKeySequenceButtonsEnabled() {
		setKeySequenceButtonsEnabled(!keyModeRandomButton.isSelected());
	}
	
	private void setKeySequenceButtonsEnabled(boolean enabled) {
		keySequenceChromaticButton.setEnabled(enabled);
		keySequenceQuartcircleButton.setEnabled(enabled);
		keySequenceQuintcircleButton.setEnabled(enabled);
	}

	private void setCadenceButtonsEnabled(boolean enabled) {
		cadenceRandomButton.setEnabled(enabled);
		cadenceSequentielButton.setEnabled(enabled);
	}
	
	private void setKeyModeButtonsEnabled(boolean enabled) {
		keyModeRandomButton.setEnabled(enabled);
		keyModeSequentialButton.setEnabled(enabled);
	}
	
	private class KeyCheckBox {
		private JCheckBox checkBox;
		private Tone tone;
		private Action keyCBAction = new AbstractAction() {
			public void actionPerformed(ActionEvent e) {
				setKeysToProvider(tone, checkBox.isSelected());
			}
		};
		
		private KeyCheckBox(Tone tone) {
			keyCBAction.putValue(Action.NAME, tone.getName());
			this.checkBox = new JCheckBox(keyCBAction);
			checkBox.setSelected(true);
			checkBox.setHorizontalTextPosition(SwingConstants.LEFT);
			this.tone = tone;
		}
		
		public JCheckBox getCheckBox() {
			return checkBox;
		}
		public Tone getTone() {
			return tone;
		}
	}
	
	private void setKeysToProvider(Tone tone, boolean selected) {
		log.debug("Tone " + tone + (selected?" ":" not ") + "selected");
		if(selected) {
			chordProvider.removeDisabledKey(tone);
		}else {
			chordProvider.addDisabledKey(tone);
		}
	}

}
