package nr.music.chordprovider.old;

import java.util.Iterator;
import java.util.Vector;

import nr.music.Cadence;
import nr.music.Chord;
import nr.music.ChordDefinition;
import nr.music.Interval;
import nr.music.Tone;
import nr.music.Tone.Sign;
import nr.music.chordprovider.CadenceProvider.Mode;
import nr.music.chordprovider.KeyProvider.KeySequence;

import org.apache.log4j.Logger;

public class ChordProvider extends AbstractChordProvider{
	static private Logger log = Logger.getLogger(ChordProvider.class);
	
	Cadence cadence = new Cadence();
	private Vector<Tone> allKeys = new Vector<Tone>();
	private Vector<Tone> currentKeys = new Vector<Tone>();
	private Vector<Chord> concretChords = new Vector<Chord>();
	private Tone providedKey;
	private Sign sign = Sign.FLAT;
	private Mode mode = Mode.SEQUENTIAL;
	private KeySequence keySequence = KeySequence.CHROMATIC; 
	
	private int nextKeyIdx;
	
	public ChordProvider() {
		initKeys(null);
	}

	public ChordProvider(Mode mode) {
		this();
		this.mode = mode;
	}
	
	public void addChordDefinition(ChordDefinition definition) {
		cadence.addChordDefinition(definition, new Interval("1"));
	}

	public void addChordDefinitions(Vector<ChordDefinition> parsedDefinitions) {
		Iterator<ChordDefinition> iter = parsedDefinitions.iterator();
		while (iter.hasNext()) {
			ChordDefinition definition = iter.next();
			addChordDefinition(definition);
		}
	}
	
	public void setCadence(Cadence cadence) {
		this.cadence = cadence;
	}

	private Tone getNextKey(boolean remove) {
		if(remove) {
			currentKeys.remove(providedKey);
			if(currentKeys.size() == 0) {
				reset();
				nextKeyIdx = 0;
			}
		}else {
			nextKeyIdx++;
			if(nextKeyIdx >= currentKeys.size()) {
				nextKeyIdx = 0;
			}
		}
		int idx = 0;
		if(mode == Mode.RANDOM) {
			int count = currentKeys.size();
			idx = (int) (Math.random() * count);
		}
		providedKey = currentKeys.elementAt(idx);
		return providedKey;
	}
	
	public void next() {
		provide(false);
	}
	
	public void provide(boolean remove) {
		if(concretChords.size() == 0) {
			concretChords = cadence.getConcretChords(getNextKey(remove));
		}
		Chord providedChord = concretChords.remove(0);
		
		
		if(providedChord != null) {
			log.debug("providing Chord:" + providedChord + " remove:" + remove);
			boolean lastChord = (concretChords.size() == 0 && currentKeys.size() == 1);
			notifyChordChanged(providedChord.clone(), lastChord);
		}else {
			log.error("no chords loaded!");
		}
	}

	public void reset() {
		currentKeys.removeAllElements();
		currentKeys.addAll(allKeys);
		concretChords.removeAllElements();
	}
	
	private void initKeys(Tone tone) {
		allKeys.clear();
		if(tone == null) {
			tone = new Tone("C");
		}
		if(sign == Sign.FLAT) {
			tone.setSign(Tone.Sign.FLAT);
		}else {
			tone.setSign(Tone.Sign.SHARP);
		}
		allKeys.add(new Tone(tone));
		int halfTones = 1;
		switch (keySequence) {
		case CHROMATIC:
			halfTones = 1;
			break;
		case QUARTCIRCLE:
			halfTones = 5;
			break;
		case QUINTCIRCLE:
			halfTones = 7;
			break;
		case RANDOM:
			//ignore
			break;
		}
		for(int i=0; i < 11; i++) {
			tone.setAbsolutHeight(tone.getAbsolutHeight() + halfTones);
			Tone newTone = new Tone(tone);
			log.debug("newTone:" + newTone);
			allKeys.add(newTone);
		}
		reset();
	}
	public Mode getCadenceProviderMode() {
		return mode;
	}
	
	protected void removeKey(Tone tone) {
		log.debug("removing key:" + tone);
		allKeys.remove(tone);
		reset();
	}

	protected void addKey(Tone tone) {
		log.debug("adding key:" + tone);
		allKeys.add(tone);
		reset();
	}

	public void setCadenceProviderMode(Mode mode) {
		this.mode = mode;
	}

	public KeySequence getKeySequence() {
		return keySequence;
	}

	public void setKeySequence(KeySequence keySequence) {
		this.keySequence = keySequence;
		initKeys(providedKey);
		provide(false);
	}

	public Sign getSign() {
		return sign;
	}

	public void setSign(Sign sign) {
		this.sign = sign;
		initKeys(providedKey);
		provide(false);
	}
}
