package nr.music.xml;

import java.io.BufferedOutputStream;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.OutputStream;
import java.io.OutputStreamWriter;
import java.io.UnsupportedEncodingException;
import java.util.Iterator;
import java.util.Vector;

import javax.xml.parsers.ParserConfigurationException;

import nr.music.ChordDefinition;
import nr.music.ChordSymbol;
import nr.music.ChordSymbol.SymbolElement;
import nr.music.ChordSymbol.SymbolRefElement;
import nr.music.Interval;

import org.apache.log4j.Logger;

public class ChordXmlWriter {
	static private Logger log = Logger.getLogger(ChordXmlWriter.class);

	File m_xmlFile;
	ChordDefinition m_chordDefinition;

	public ChordXmlWriter() {
	}

	public ChordXmlWriter(File xmlFile, ChordDefinition chordDefinition) {
		this.m_xmlFile = xmlFile;
		this.m_chordDefinition = chordDefinition;
	}

	public int write() throws ParserConfigurationException {

		try {
			OutputStream fout = new FileOutputStream(this.m_xmlFile);
			OutputStream bout = new BufferedOutputStream(fout);
			OutputStreamWriter out = new OutputStreamWriter(bout, "8859_1");

			out.write("<?xml version=\"1.0\" ");
			out.write("encoding=\"ISO-8859-1\"?>\r\n");
			out.write("<!--DOCTYPE properties SYSTEM \"http://java.sun.com/dtd/properties.dtd\"-->\r\n");

			out.write("<chords label=\"" + m_chordDefinition.getName() + "\">"
					+ "\r\n");
			out.write("<chordsymbol name=\"" + m_chordDefinition.getName()
					+ "\">" + "\r\n");
			ChordSymbol mySymbol = new ChordSymbol(new String());
			mySymbol = m_chordDefinition.getSymbol();

			Iterator<SymbolElement> iter = mySymbol.getM_symbolElements()
					.iterator();
			String mySmallString;
			String myTextString;

			while (iter.hasNext()) {
				SymbolElement symbolElement = iter.next();

				if (symbolElement.isSmallTextSize()) {
					mySmallString = new String(" size=\"small\"/>");
				} else {
					mySmallString = new String("/>");
				}

				if (symbolElement instanceof SymbolRefElement) {
					SymbolRefElement symbolRefElement = (SymbolRefElement) symbolElement;
					myTextString = "textref=\"" + symbolRefElement.m_text
							+ "\"";
				} else {
					myTextString = "text=\"" + symbolElement.getText() + "\"";
				}

				switch (symbolElement.getLine()) {
				case BASE:
					out.write("<baseline " + myTextString + mySmallString
							+ "\r\n");
					break;
				case UPPER:
					out.write("<upperline " + myTextString + mySmallString
							+ "\r\n");
					break;
				}
			}
			out.write("</chordsymbol>\r\n");

			// write node <chorddefinition>
			out.write("<chorddefinition name=\"" + m_chordDefinition.getName()
					+ "\" symbol=\"" + m_chordDefinition.getName() + "\">"
					+ "\r\n");

			// loop over given intervals
			Vector<Interval> intervals = new Vector<Interval>();
			intervals = m_chordDefinition.getM_intervals();

			for (int i = 0; i < intervals.size(); i++) {
				out.write("<interval name=\"" + intervals.get(i).getHalfTones()
						+ "\"/>" + "\r\n");
			}

			out.write("</chorddefinition>" + "\r\n");

			// write node <cadence>
			out.write("<cadence>" + "\r\n");
			out.write("<chorddefinitionref ref=\""
					+ m_chordDefinition.getName() + "\" stufe=\"1\">" + "\r\n");

			// loop over given inversions
			Vector<Integer> inversions = new Vector<Integer>();
			inversions = m_chordDefinition.getM_allowedInversions();

			// Attention: here FIX two inversions added!! TODO -> fill in
			// ConfPanel
			inversions.add(new Integer(0));
			inversions.add(new Integer(2));

			for (int i = 0; i < inversions.size(); i++) {
				out.write("<inversion no=\"" + inversions.get(i) + "\"/>"
						+ "\r\n");
			}

			out.write("</chorddefinitionref>" + "\r\n");
			out.write("</cadence>" + "\r\n");

			out.write("</chords>\r\n");

			// here is the end of the written xml file

			out.flush(); // Don't forget to flush!
			out.close();
		} catch (UnsupportedEncodingException e) {
			log.error("This VM does not support the Latin-1 character set.", e);
		} catch (IOException e) {
			log.error(e.getMessage(), e);
		}

		return 0;
	}
}
